#!/bin/bash
set -x

usage () {
    echo "USAGE"
    echo "$1 [-d|-dd] [-p <prefix>]"
    echo "  -dd dry totally run (local and system wide)"
    echo "  -d  dry run system-related commands"
    echo "  -p  prefix path for the installation (default is ./out)"
    echo "  -t  produce a tar archive of the output (use it with fakeroot)"
    exit $2
}

#
#  MAIN
#
GROUPNAME=threegates

DRYRUN=0
EXCMD_FS=""
EXCMD_SY=""
THREEGATES_BASE="./out"
WITH_ARCHIVE=0

while [ $# -gt 0 ]; do
    case $1 in
        -dd) DRYRUN=2 ;;
        -d)  DRYRUN=1 ;;
        -p)  THREEGATES_BASE="$2" ; shift ;;
        -t)  WITH_ARCHIVE=1 ;;
        *)   usage $0  ;;
    esac
    shift
done

if [ $DRYRUN -eq 2 ]; then
    EXCMD_FS="echo "
fi
if [ $DRYRUN -ge 1 ]; then
    EXCMD_SY="echo "
fi

if ! grep -q $GROUPNAME /etc/group ; then
    $EXCMD_SY groupadd -r $GROUPNAME
fi

$EXCMD_FS mkdir -p        ${THREEGATES_BASE}/usr/lib/threegates/bin
$EXCMD_FS cp raw/root/usr/bin/crl-manager.py ${THREEGATES_BASE}/usr/bin
$EXCMD_SY chown root.root ${THREEGATES_BASE}/usr/bin/crl-manager.py
$EXCMD_FS chmod 744       ${THREEGATES_BASE}/usr/bin/crl-manager.py


$EXCMD_FS mkdir -p  ${THREEGATES_BASE}/var/lib/threegates
$EXCMD_FS chmod 755 ${THREEGATES_BASE}/var/lib/threegates

$EXCMD_FS mkdir -p  ${THREEGATES_BASE}/var/lib/threegates/ca
$EXCMD_FS chmod 755 ${THREEGATES_BASE}/var/lib/threegates/ca

$EXCMD_FS mkdir -p  ${THREEGATES_BASE}/var/lib/threegates/crl
$EXCMD_FS chmod 755 ${THREEGATES_BASE}/var/lib/threegates/crl

$EXCMD_FS mkdir -p  ${THREEGATES_BASE}/var/lib/threegates/crluri
$EXCMD_FS chmod 755 ${THREEGATES_BASE}/var/lib/threegates/crluri

$EXCMD_FS mkdir -p  ${THREEGATES_BASE}/var/lib/threegates/crlurinew
$EXCMD_FS chmod 775 ${THREEGATES_BASE}/var/lib/threegates/crlurinew
$EXCMD_FS chmod g+s ${THREEGATES_BASE}/var/lib/threegates/crlurinew
$EXCMD_SY chgrp $GROUPNAME ${THREEGATES_BASE}/var/lib/threegates/crlurinew

export THREEGATES_CA_BASE="${THREEGATES_BASE}/var/lib/threegates/ca"
$EXCMD_FS ./bin/certs_extractor.sh

if [ $WITH_ARCHIVE -eq 1 ]; then
    cd $THREEGATES_BASE
    tar zcvf ../threegates_out.tgz *
fi

exit 0
